const Generator = require('yeoman-generator');
const { EOL } = require('os');

module.exports = class extends Generator {
  writing() {
    // Update src/index.js
    const index = this.fs.read(this.destinationPath('src/index.js'));
    const amplifyImport = `${EOL}import Amplify from 'aws-amplify';`;
    const authenticatorImport = `${EOL}import { Authenticator } from 'aws-amplify-react';`;

    let amplifyImportDone = false;
    let authenticatorImportDone = false;
    let amplifyConfigDone = false;

    let newIndex = index.toString().split(EOL).map((line) => {
      if (line.match(/import/) && !amplifyImportDone && !authenticatorImportDone) {
        line += amplifyImport;
        line += authenticatorImport;

        amplifyImportDone = true;
        authenticatorImportDone = true;
      }

      if (line.match(/ReactDOM\.render/) && !amplifyConfigDone) {
        const lastLine = line;

        line = `Amplify.configure({${EOL}`;
        line += `  Auth: {${EOL}`;
        line += `    identityPoolId: '',${EOL}`;
        line += `    mandatorySignIn: true,${EOL}`;
        line += `    region: '',${EOL}`;
        line += `    userPoolId: '',${EOL}`;
        line += `    userPoolWebClientId: '',${EOL}`;
        line += `  },${EOL}`;
        line += `  Storage: {${EOL}`;
        line += `    bucket: '',${EOL}`;
        line += `    identityPoolId: '',${EOL}`;
        line += `    region: '',${EOL}`;
        line += `  },${EOL}`;
        line += `});${EOL}`;
        line += EOL;

        line += lastLine;

        amplifyConfigDone = true;
      }

      return line;
    }).join(EOL);

    newIndex = newIndex.replace(/<App \/>/, '<Authenticator><App /></Authenticator>');

    this.fs.write(this.destinationPath('src/index.js'), newIndex);

    // Update src/App.js
    let App = this.fs.read(this.destinationPath('src/App.js'));

    App = App.replace(
      /<Route path='\/'(.*)\/>/,
      '{this.props.authState === \'signedIn\' && <Route path=\'/\'$1/>}'
    );

    this.fs.write(this.destinationPath('src/App.js'), App);
  }

  install() {
    this.npmInstall([
      'aws-amplify',
      'aws-amplify-react',
    ]);
  }
};

